from src.tools import feature_maker
from src.utility import data_dto
import pandas as pd
import numpy as np
import datetime
from typing import NamedTuple

def test_get_start_year():
    Screening = NamedTuple('Screening', [('ScreeningDate', str)])
    sc1 = Screening(ScreeningDate=np.array('2020-07-20'))
    res = feature_maker.get_start_year(sc1)
    assert res == 2020

def test_get_birth_year():
    sc = pd.DataFrame({'BirthYear': ['57']})
    res = feature_maker.get_birth_year(sc)
    assert res == 57

def test_get_gender():
    sc = pd.DataFrame({'Gender': ['FEMALE']})
    res = feature_maker.get_gender(sc)
    assert res == 'Female'

def test_get_citizen_data():
    sc = pd.DataFrame({'CitizenId': ['455910', '455911']})
    ss = pd.DataFrame({'CitizenId': ['455910', '455911']})
    td = pd.DataFrame({'CitizenId': ['455910', '455911']})
    tc = pd.DataFrame({'CitizenId': ['455910', '455911']})
    ats = pd.DataFrame({'CitizenId': ['455910', '455911']})
    fd = pd.DataFrame({'CitizenId': ['455910', '455911']})
    df = data_dto.Data(sc, ss, td, tc, ats, fd)
    res = feature_maker.get_citizen_data(df, '455910')
    assert res.sc.shape[0] == 1
    assert res.ss.shape[0] == 1
    assert res.td.shape[0] == 1
    assert res.tc.shape[0] == 1
    assert res.ats.shape[0] == 1
    assert res.fd.shape[0] == 1

def test_get_window_data():
    td = pd.DataFrame({'RatingDate': ['2020-07-10', '2020-07-15']})
    tc = pd.DataFrame({'RatingDate': ['2020-07-10', '2020-07-15']})
    ss = pd.DataFrame({'ChangeDate': ['2020-07-10', '2020-07-15']})
    start_date = '2020-07-09'
    end_date = '2020-07-11'
    tdw, tcw, ssw = feature_maker.get_screening_data(td, tc, ss, start_date, end_date)
    assert tdw.shape[0] == 1
    assert tcw.shape[0] == 1
    assert ssw.shape[0] == 1

def test_convert_date_to_datetime():
    string_date = '01-01-2019'
    datetime_date = datetime.datetime(2019, 1, 1)
    date_format = '%d-%m-%Y'
    res = feature_maker.convert_date_to_datetime(string_date, date_format)
    assert isinstance(res, datetime.date)
    assert res.ctime() == datetime_date.ctime()

def test_get_cancels_week():
    timestamp_date = pd.Timestamp(datetime.datetime(2020,7,10))
    tcw = pd.DataFrame({'RatingDate': [timestamp_date]})
    res = feature_maker.get_cancels_week(tcw)
    assert res.item() == "28/2020"

def test_get_cluster():
    cl = pd.DataFrame({'Cluster': [42]})
    res = feature_maker.get_cluster(cl)
    assert res == 42

def test_get_interval_length():
    start_date = pd.Timestamp(datetime.datetime(2020,7,10))
    end_date = pd.Timestamp(datetime.datetime(2020,7,31))
    res = feature_maker.get_interval_length(start_date, end_date)
    assert isinstance(res, float)
    assert res == 3

def test_get_rehab_indicator():
    needs_start = 10
    physics_start = 5
    res =  feature_maker.get_rehab_indicator(needs_start, physics_start)
    assert res == 2

def test_has_device_id():
    devices = ['120606']
    res = feature_maker.has_ats_id('120606', devices)
    assert res

def test_get_n_cancel_week_min():
    cancels = pd.Series(['10/2020', '10/2020', '11/2020', '12/2020'])
    res = feature_maker.get_n_cancel_week_min(cancels)
    assert res == 1

def test_get_time_between_training_mean():
    cancel_date1 = pd.Timestamp(datetime.datetime(2020,7,10))
    cancel_date2 = pd.Timestamp(datetime.datetime(2020,7,15))
    tcw = pd.DataFrame({'RatingDate': [cancel_date1, cancel_date2]})
    res = feature_maker.get_time_between_training_mean(tcw)
    assert res == 5

def get_mean_cancels_week():
    n_cancel = 10
    n_weeks = 5
    res = feature_maker.get_mean_cancels_week(n_cancel, n_weeks)
    assert res == 2

def test_get_n_weeks_without_training():
    n_weeks = 10
    n_weeks_with_trainings = 2
    res = feature_maker.get_n_weeks_without_training(n_weeks, n_weeks_with_trainings)
    assert res == 8

def test_get_n_weeks_with_training():
    rating_date = pd.Timestamp(datetime.datetime(2020,7,19))
    tdw = pd.DataFrame({'RatingDate': [rating_date]})
    start_date = pd.Timestamp(datetime.datetime(2020,7,12))
    res = feature_maker.get_n_weeks_with_training(tdw, start_date)
    assert res == 1

def test_get_training_week():
    rating_date = pd.Timestamp(datetime.datetime(2020,7,19))
    tdw = pd.DataFrame({'RatingDate': [rating_date]})
    start_date = pd.Timestamp(datetime.datetime(2020,7,12))
    res = feature_maker.get_training_week(tdw, start_date)
    assert res.item() == 1

def test_get_n_training_week_min():
    training_per_week = pd.Series([3, 3, 2])
    res = feature_maker.get_n_training_week_min(training_per_week, 6, 3)
    assert res == 1

def test_get_n_training_week_max():
    training_per_week = pd.Series([3, 3, 2])
    res = feature_maker.get_n_training_week_max(training_per_week)
    assert res == 2

def test_get_n_training_window():
    rating_date = pd.Timestamp(datetime.datetime(2020,7,19))
    tdw = pd.DataFrame({'RatingDate': [rating_date]})
    res = feature_maker.get_n_training_window(tdw)
    assert res == 1

def test_get_n_trainin_week():
    n_weeks = 10
    n_training_window = 2
    res = feature_maker.get_n_training_week(n_weeks, n_training_window)
    assert res == 0.2

def test_get_n_train_optimal():
    n_weeks = 10
    res = feature_maker.get_n_training_optimal(n_weeks)
    assert res == 20

def test_get_max_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = feature_maker.get_max_evaluation(tdw)
    assert res == 3

def test_get_min_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = feature_maker.get_min_evaluation(tdw)
    assert res == 1

def test_get_std_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = feature_maker.get_std_evaluation(tdw)
    assert res == 1

def test_get_mean_evaluation():
    tdw = pd.DataFrame({'RatingScore': [1, 2, 3]})
    res = feature_maker.get_mean_evaluation(tdw)
    assert res == 2

def test_get_time_between_training_mean():
    rating_date1 = pd.Timestamp(datetime.datetime(2020,7,19))
    rating_date2 = pd.Timestamp(datetime.datetime(2020,7,22))
    rating_date3 = pd.Timestamp(datetime.datetime(2020,7,25))
    tdw = pd.DataFrame({'RatingDate':
     [rating_date1, rating_date2, rating_date3]})
    res = feature_maker.get_time_between_training_mean(tdw)
    assert res == 3

def test_get_needs_reason():
    Screening = NamedTuple('Screening', [('NeedForHelpReason', str)])
    sc1 = Screening(NeedForHelpReason=np.array('Reason'))
    res = feature_maker.get_needs_reason(sc1)
    assert res == 'Reason'

def test_get_physics_reason():
    Screening = NamedTuple('Screening', [('PhysicalStrengthReason', str)])
    sc1 = Screening(PhysicalStrengthReason=np.array('Reason'))
    res = feature_maker.get_physics_reason(sc1)
    assert res == 'Reason'

def test_get_last_status():
    ssw = pd.DataFrame({'Status': ['Status']})
    res = feature_maker.get_last_status(ssw)
    assert res == 'Status'

def test_get_last_status_date():
    end_date = '2018-12-12'
    ssw = pd.DataFrame({'ChangeDate': ['01-01-2019', '02-01-2019']})
    date_format = '%d-%m-%Y'
    res = feature_maker.get_last_status_date(ssw, end_date, date_format)
    assert res == pd.to_datetime('02-01-2019', format=date_format)

def test_get_exercises():
    ec = '001,002,003'
    res = feature_maker.get_exercises(ec)
    assert res == ec

def test_get_physics_indicator_positive():
    res = feature_maker.get_physics_indicator(41, 42)
    assert res == 1

def test_get_physics_indicator_negative():
    res = feature_maker.get_physics_indicator(42, 41)
    assert res == 0